#pragma GCC optimize("O3")
#pragma GCC target("sse4.1")

#include <bits/stdc++.h>
using namespace std;

using T = double;
const T prec = 1e-6;
const T log_bound = T(logl(1 - prec));

int main ()
{
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);

    int n;
    cin >> n;

    vector<vector<int>> costs(n, vector<int>(n + 1));
    for (int i = 0; i < n; i++)
    for (int j = 1; j <= n; j++)
        cin >> costs[i][j];

    vector<T> logfact(n + 1);
    for (int i = 1; i <= n; i++)
        logfact[i] = logfact[i - 1] + logl(i);

    auto log_prob_not_loss = [&] (int x, int y)
    {
        // definitely not a loss
        if (x + y > n)
            return T(0);

        // a loss with probability (n - x)! (n - y)! / (n! (n - x - y)!)
        double log_loss = logfact[n - x] + logfact[n - y] - logfact[n] - logfact[n - x - y];
        return T(logl(1.0 - expl(log_loss)));
    };

    vector<int> at_level(n + 1, 1);

    {
        int sum = 1, cnt = 1;
        T log_not_loss = 0;
        for (int level = n - 1; level >= 1; level--)
        {
            const T expect_log = (n - level) * log_bound / (n - 1);
            while (true)
            {
                const T cur_log = log_prob_not_loss(level, cnt);
                if (log_not_loss + cur_log >= expect_log)
                {
                    log_not_loss += cur_log;
                    sum += cnt;
                    break;
                }
                else
                    cnt++;
            }

            at_level[level] = cnt;
        }
    }

    vector<int> heights(n);
    {
        for (int i = 1; i <= n; i++)
        for (int j = 0; j < at_level[i]; j++)
            heights[j]++;
    }

    mt19937 rng;
    shuffle(heights.begin(), heights.end(), rng);

    const int inf = int(1e9);
    vector<int> ans(n + 1, inf);
    vector<vector<int>> best_cost(n + 1, vector<int>(n + 1, inf));

    for (int start = 0; start < n; start++)
    {
        const int h = heights[start];
        for (int i = 0; i <= h; i++)
            fill(best_cost[i].begin(), best_cost[i].end(), inf);

        best_cost[0][0] = 0;
        for (int i = 0; i < h; i++)
        for (int j = i; j < n; j++)
        {
            const int from = best_cost[i][j];
            int *best_cost_ptr = &best_cost[i + 1][j + 0];
            int *cost_ptr = &costs[(j + start) % n][0];

            const int block = 8;
            int rem = (n - j) % block;
            int full = (n - j) / block;
#define op {best_cost_ptr++, cost_ptr++; *best_cost_ptr = (*best_cost_ptr < *cost_ptr + from ? *best_cost_ptr : *cost_ptr + from);}
            while (rem--)
                op;
            while (full--)
            {
                op; op; op; op;
                op; op; op; op;
            }
        }

        for (int i = 0; i <= h; i++)
            ans[i] = min(ans[i], best_cost[i][n]);
    }

    for (int i = 1; i <= n; i++)
        cout << ans[i] << ' ';
    cout << endl;
}
